% Simple LPV Example
% RNC Paper Pfifer & Seiler


% Describe constant scalar uncertainty by IQC
npsi = 3; %number of IQC multiplier
kappa = 0.5; %norm bound on Delta
Hs{1} = ss(1);
Hs{2} = ss(tf(1,[1,1.5]));
Hs{3} = ss(tf(.5,[1 1]));

%initialization of variables
Psiup = []; Psilow= [];
%Build filter Psi of the IQC factorization
for i=1:npsi
    Psiup = [Psiup;  [kappa*Hs{i}, 0]];
    Psilow = [Psilow;  [0, Hs{i}]];
end
Psi = [Psiup;Psilow];

M = zeros(2*npsi,2*npsi); %initialize M (only size important
IQClist = {{Psi,M}};


% bounds on the parameter variation rates
RateBounds = [-1,1];

% basis function for the parameter dependent lyapunov function
Fbasis = ones(2,1,6); %First term is constant
Fbasis(2,1,:) = reshape(yvec',1,1,6); %second term linear in rho
%gradient of the basis function with respect to the parameters
Fgrad = zeros(2,1,6);
Fgrad(2,1,:) = ones(1,1,6);



FlgPos = 1; % adds P >=0 constraint 
[gam1,info1] = simplenormboundedrealgain(G,IQClist,Fbasis,Fgrad,RateBounds,FlgPos);

FlgPos = 0; % does not include P >= 0 constraint
[gam2,info2] = simplenormboundedrealgain(G,IQClist,Fbasis,Fgrad,RateBounds,FlgPos);

disp([gam1, gam2])

%% Check that there is a hard factorization of the generalized parameterization
% (problem 2 above)

%Obtain optimal IQC paramterisation
M = info2.Mopt;
Apsi = IQClist{1}{1}.a;
Bpsi = IQClist{1}{1}.b;
Cpsi = IQClist{1}{1}.c;
Dpsi = IQClist{1}{1}.d;

% Compute j-spectral factorization of optimal IQC parameterisation by
% solving ARE
Q = Cpsi'*M*Cpsi;
S = Cpsi'*M*Dpsi;
R = Dpsi'*M*Dpsi;

[U,T] = schur(R);
ev = real( diag(T) );
pidx = find( ev>0 );
qidx = find( ev<0 );
W = diag( sqrt([ev(pidx); -ev(qidx)]) );
W = W*U(:,[pidx;qidx])';
p = length(pidx);
q = length(qidx);
Jpq = blkdiag( eye(p), -eye(q) );
X0 = care(Apsi,Bpsi,Q,R,S);
% Psi is the filter of the resulting hard factoization
Psi = ss(Apsi,Bpsi,Jpq*inv(W')*(Bpsi'*X0+S'),W);

% test that the modified Lyapunov matrix is positiv definite whereas the 
% original one is not
rho = 2;
Ptest = info2.P(:,:,1)+ rho*info2.P(:,:,2);
Ptilde = Ptest - blkdiag(X0,zeros(4,4));
eig(Ptest)
eig(Ptilde)